/**
 ** mhpCntrlTaskCodels.cc
 **
 ** Codels used by the control task mhpCntrlTask
 **
 ** Author: E. Akin Sisbot
 ** Date: Nov 06, update Aug 09
 **
 **/

#include <portLib.h>

#include "server/mhpHeader.h"
#include "mhpm3d.h"
#include <libmove3d/hri/hri_distance.h>

/*------------------------------------------------------------------------
 * mhpDebug_modeCntrl  -  control codel of CONTROL request Debug_mode
 *
 * Description:
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
mhpDebug_modeCntrl(int *debug_mode, int *report)
{
  SDI_F->State.debug_mode = *debug_mode;

  return OK;
}

/*------------------------------------------------------------------------
 * ArmStopPlanning
 *
 * Description:    
 * Report: OK
 */

STATUS
mhpArmStopPlanningCntrl(GEN_BOOL* stopPlanning, int *report)
{
  int value = ( *stopPlanning == GEN_TRUE );
  printf( "Set stop value to : %d\n", value );

  p3d_SetStopValue( value );
#ifdef HRI_COSTSPACE
  PlanEnv->setBool( PlanParam::stopPlanner, value );
#endif
  return OK;
}


STATUS
mhpArmInterruptCollisionOnTrajCntrl(int *report)
{
  SDI_F->collisionState.state = GENMANIP_COLLISION_FREE;
  SDI_F->State.planning = FALSE;

  return OK;
}

/*------------------------------------------------------------------------
 * mhpInterfaceAgenParamsCtrl  -  control codel of CONTROL request SetInterfaceAgentParams
 *
 * Description:    
 * Report: OK
 */

STATUS
mhpInterfaceAgenParamsCtrl(MHP_INTERFACE_AGENT_PARAMS *InterfaceAgentParams, int *report)
{
  int i;

  for(i=0; i<MHP_AGENTS->all_agents_no; i++) {
    if(strcasestr(MHP_AGENTS->all_agents[i]->robotPt->name, InterfaceAgentParams->agent_name.name)){
      if(InterfaceAgentParams->show_fov) {
	MHP_AGENTS->all_agents[i]->perspective->enable_vision_draw = TRUE;
      }
      else {
	MHP_AGENTS->all_agents[i]->perspective->enable_vision_draw = FALSE;	
      } 
      
      if(InterfaceAgentParams->show_poiting) {
	MHP_AGENTS->all_agents[i]->perspective->enable_pointing_draw = TRUE;
      }
      else {
	MHP_AGENTS->all_agents[i]->perspective->enable_pointing_draw = FALSE;
      }
    }
  }
  
  return OK;  
}

/*------------------------------------------------------------------------
 * mhpSetCollisionWithHumanArmsCntrl  -  control codel of CONTROL request SetCollisionWithHumanArms
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS 
mhpSetCollisionCheckWithHumanArmsCntrl(GEN_BOOL *collisionHumanArms, int *report)
{
  bool setCollision = ( *collisionHumanArms == GEN_TRUE );

  HRI_AGENT* robot = MHP_AGENTS->all_agents[ MHP_AGENTS->source_agent_idx ];

  if( !hri_activate_coll_robot_and_all_humans_arms( robot, MHP_AGENTS , setCollision ) )
    {
      printf("Problem to cancel collision checking with arms\n");
      return ERROR;
    }

  printf("Set collision between humans arms and MHP_ROBOT\n"); 
  return OK;
}

/*------------------------------------------------------------------------
 * mhpSetCollisionCheckWithObjectCntrl  -  control codel of CONTROL request SetCollisionCheckWithObject
 *
 * Description:    
 * Report: OK
 *
 * Returns:    OK or ERROR
 */

STATUS
mhpSetCollisionCheckWithObjectCntrl(MHP_OBJECT_NAME *collisionObject, int *report)
{
   bool setCollision = ( collisionObject->mode == GEN_TRUE );

   HRI_AGENT* robot = MHP_AGENTS->all_agents[ MHP_AGENTS->source_agent_idx ];

   if( !hri_activate_coll_robot_and_object( robot, collisionObject->objectName.name, setCollision ) )
    {
      printf("Problem to cancel collision checking with object\n");
      return ERROR;
    }

   printf("Set collision between object and MHP_ROBOT\n");
   return OK;
}
